#ifndef _MODIFIERAPI_H_
#define _MODIFIERAPI_H_

/*-------------------------------------------------------------------------------------------------------
												ModifierApi 1.2

PSM_MEMORY_BLOCK function :

A PSP_MEMORY_BLOCK is a memory block allocated by pogoshell studio that is passed to the modifier, it's
mainly used to pass the data to modify and save/load the modifier data to the psw file.
To use a memory block, you need to lock it with MemoryBlock_Lock that will return a memory pointer, once
you are done, you need to call MemoryBlock_Unlock to let pogoshell studio access the memory again, if you
trash the memory, MemoryBlock_Unlock will Assert.
If you need to allocate more or less memory, call MemoryBlock_Resize with the new size, the memory block
need to be unlocked to do that, the pointer returned by lock might change after that, so don't store it
In addition of the of the random access to the memory trough the pointer returned by MemoryBlock_Lock,
you can access to the memory like you would do with a file through all the Read/Write function and 
MemoryBlock_SetPos, MemoryBlockGetPos, but the file still need to be locked to do that.
The version 1.2 of the Api introduce the concept of PSM_READONLY_MEMORY_BLOCK, it's a memory block where
data can only be read from. All the function that need a PSM_READONLY_MEMORY_BLOCK work as well with a 
PSM_MEMORY_BLOCK, but all the function that need a PSM_MEMORY_BLOCK doesn't work with a PSM_READONLY_MEMORY_BLOCK

bool			MemoryBlock_Resize		(PSM_MEMORY_BLOCK psmb,size_t nNewSize);
size_t			MemoryBlock_GetSize		(PSM_READONLY_MEMORY_BLOCK psmb);
void		*	MemoryBlock_Lock		(PSM_MEMORY_BLOCK psmb);
const void	*	MemoryBlock_Lock		(PSM_READONLY_MEMORY_BLOCK psmb);
bool			MemoryBlock_UnLock		(PSM_READONLY_MEMORY_BLOCK psmb);
size_t			MemoryBlock_GetPos		(PSM_READONLY_MEMORY_BLOCK psmb);
bool			MemoryBlock_SetPos		(PSM_READONLY_MEMORY_BLOCK psmb,int n);
int				MemoryBlock_ReadInt		(PSM_READONLY_MEMORY_BLOCK psmb);
float			MemoryBlock_ReadFloat	(PSM_READONLY_MEMORY_BLOCK psmb);
bool			MemoryBlock_ReadBool	(PSM_READONLY_MEMORY_BLOCK psmb);
size_t			MemoryBlock_ReadString	(PSM_READONLY_MEMORY_BLOCK psmb,char * pszString, size_t nSize);
bool 			MemoryBlock_WriteInt	(PSM_MEMORY_BLOCK psmb,int n);
bool 			MemoryBlock_WriteFloat	(PSM_MEMORY_BLOCK psmb,float v);
bool 			MemoryBlock_WriteBool	(PSM_MEMORY_BLOCK psmb,bool b);
bool 			MemoryBlock_WriteString	(PSM_MEMORY_BLOCK psmb,const char * pszString);


Config function:

The config function let you write/read data to the config file, the section is the string between [], for
example [Modifiers Settings], don't hesitate to create your own modifier section. The key is the string
before the =, for example IpsPatcher in IpsPatcher = "D:\Gba\Pogoshell studio\Modifiers\Ips Patcher.psm"
The key can't have space. The value is the string after the = so here it's 
D:\Gba\Pogoshell studio\Modifiers\Ips Patcher.psm , the quote are had automaticly if you have space in
your value.
Config_DeleteSection delete a section and all the key inside, so please use it only on the section you 
created for your own modifier.

void			Config_WriteKey			(const char * pszSection, const char * pszKey, const char * pszValue);
void			Config_WriteKey			(const char * pszSection, const char * pszKey, int nValue);
void			Config_WriteKey			(const char * pszSection, const char * pszKey, bool bValue);
const char *	Config_GetKey			(const char * pszSection, const char * pszKey, const char * pszDefaultValue);
int				Config_GetKey			(const char * pszSection, const char * pszKey, int nDefaultValue);
bool			Config_GetKey			(const char * pszSection, const char * pszKey, bool bDefaultValue);
void			Config_DeleteSection	(const char * pszSection);


Output function:

Those function let you print stuff in the output windows

void Output_Printf	(const char * pszString,...);
void Output_Printfc	(bool bBold,COLORREF clrText,const char * pszString,...);

Psf (Pogoshell studio file) function:

A HPSF (handle on a pogoshell studio file) let you access directly to a file within the compilation. 
Psf_OpenOriginalFile will give you access to the content of the original file as it is on the hard drive
or in the zip file (aka : not modifier by the modifier). Don't forget to close the file with
Psf_CloseOriginalFile once you don't need this memory anymore to avoid memory leek

const char *				Psf_GetName				(HPSF hFile);
bool						Psf_Rename				(HPSF hFile,const char * pszNewName);
const char *				Psf_GetExtension		(HPSF hFile);
PSM_READONLY_MEMORY_BLOCK	Psf_OpenOriginalFile	(HPSF hFile);
void						Psf_CloseOriginalFile	(HPSF hFile,PSM_READONLY_MEMORY_BLOCK psmb);

PSMVERSION function:

Those function let you create version data, and get the Minor and Major version of a PSMVERSION

int			GetPsmMinorVersion	(PSMVERSION Version);
int			GetPsmMajorVersion	(PSMVERSION Version);
PSMVERSION	MakePsmVersion		(int nMajor, int nMinor);

---------------------------------------------------------------------------------------------------------*/

#include <Windows.h>
#include <stdio.h>

#define EXPORT_PSM extern "C" __declspec(dllexport)

typedef unsigned int HPSMI;			//Handle of a modifier instance
const HPSMI	INVALID_INSTANCE = NULL;

class CReadOnlyPSM_MEMORY_BLOCK {};
class CPSM_MEMORY_BLOCK : public CReadOnlyPSM_MEMORY_BLOCK {};
typedef void * HPSF;							//Handle on a Pogoshell studio file
typedef unsigned int PSMVERSION;				//Version information
typedef CPSM_MEMORY_BLOCK * PSM_MEMORY_BLOCK;	//Modifier memory block, please don't do any delete or free on it
typedef CReadOnlyPSM_MEMORY_BLOCK * PSM_READONLY_MEMORY_BLOCK;	//Modifier Read only memory block, please don't do any delete or free on it

inline int GetPsmMinorVersion(PSMVERSION Version)
{
	int MinorVersion = Version & 0x0000FFFF;
	while (MinorVersion % 10 == 0 && MinorVersion != 0)
		MinorVersion /= 10;
	return MinorVersion;
}

inline int GetPsmMajorVersion(PSMVERSION Version)
{ 
	return Version >> 16;
}

inline PSMVERSION MakePsmVersion(int nMajor, int nMinor) 
{	
	while (nMinor != 0 && nMinor < 10000)
		nMinor *= 10;

	return (nMajor << 16) | nMinor;
}

static const PSMVERSION MODIFIER_API_VERSION = MakePsmVersion(1,2);

#ifndef _MAIN_EXE_

inline bool MemoryBlock_Resize(PSM_MEMORY_BLOCK psmb,size_t nNewSize)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK,size_t);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_Resize");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,nNewSize);
}

inline size_t MemoryBlock_GetSize(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef size_t(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_GetSize");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline void * MemoryBlock_Lock(PSM_MEMORY_BLOCK psmb)
{
	typedef void *(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_Lock");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline const void * MemoryBlock_Lock(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef const void *(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_LockReadOnly");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline bool MemoryBlock_UnLock(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_UnLock");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline size_t MemoryBlock_GetPos(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef size_t(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_GetPos");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline bool MemoryBlock_SetPos(PSM_READONLY_MEMORY_BLOCK psmb, int n)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK,int);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_SetPos");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,n);
}

inline int MemoryBlock_ReadInt(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef int(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_ReadInt");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline float MemoryBlock_ReadFloat(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef float(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_ReadFloat");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline bool MemoryBlock_ReadBool(PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_ReadBool");
	return (*pFunc)(MODIFIER_API_VERSION,psmb);
}

inline size_t MemoryBlock_ReadString(PSM_READONLY_MEMORY_BLOCK psmb,char * pszString, size_t nSize)
{
	typedef size_t(*FnPtr)(PSMVERSION,PSM_READONLY_MEMORY_BLOCK,char *,size_t);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_ReadString");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,pszString,nSize);
}

inline bool MemoryBlock_WriteInt(PSM_MEMORY_BLOCK psmb, int n)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK,int);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_WriteInt");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,n);
}

inline bool MemoryBlock_WriteFloat(PSM_MEMORY_BLOCK psmb,float v)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK,float);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_WriteFloat");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,v);
}

inline bool MemoryBlock_WriteBool(PSM_MEMORY_BLOCK psmb,bool b)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK,bool);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_WriteBool");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,b);
}

inline bool MemoryBlock_WriteString(PSM_MEMORY_BLOCK psmb,const char * pszString)
{
	typedef bool(*FnPtr)(PSMVERSION,PSM_MEMORY_BLOCK,const char *);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"MemoryBlock_WriteString");
	return (*pFunc)(MODIFIER_API_VERSION,psmb,pszString);
}

inline void Config_WriteKey(const char * pszSection, const char * pszKey, const char * pszValue)
{
	typedef void(*FnPtr)(PSMVERSION,const char *,const char *,const char *);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_WriteStringKey");
	(*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,pszValue);
}

inline void Config_WriteKey(const char * pszSection, const char * pszKey, int nValue)
{
	typedef void(*FnPtr)(PSMVERSION,const char *,const char *,int);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_WriteIntKey");
	(*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,nValue);
}

inline void Config_WriteKey(const char * pszSection, const char * pszKey, bool bValue)
{
	typedef void(*FnPtr)(PSMVERSION,const char *,const char *,bool);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_WriteBoolKey");
	(*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,bValue);
}

inline const char * Config_GetKey(const char * pszSection, const char * pszKey, const char * pszDefaultValue)
{
	typedef const char *(*FnPtr)(PSMVERSION,const char *,const char *,const char *);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_GetStringKey");
	return (*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,pszDefaultValue);
}

inline int Config_GetKey(const char * pszSection, const char * pszKey, int nDefaultValue)
{
	typedef int(*FnPtr)(PSMVERSION,const char *,const char *,int);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_GetIntKey");
	return (*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,nDefaultValue);
}

inline bool Config_GetKey(const char * pszSection, const char * pszKey, bool bDefaultValue)
{
	typedef bool(*FnPtr)(PSMVERSION,const char *,const char *,bool);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_GetBoolKey");
	return (*pFunc)(MODIFIER_API_VERSION,pszSection,pszKey,bDefaultValue);
}

inline void Config_DeleteSection(const char * pszSection)
{
	typedef void(*FnPtr)(PSMVERSION,const char *);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Config_DeleteSection");
	(*pFunc)(MODIFIER_API_VERSION,pszSection);
}

inline void Output_Printf(const char * pszString,...)
{
	va_list args;
	va_start(args,pszString);
	typedef void(*FnPtr)(PSMVERSION,bool,COLORREF,const char *,va_list);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Output_Printfc");
	(*pFunc)(MODIFIER_API_VERSION,false,RGB(0,0,0),pszString,args);
	va_end(args);	
}

inline void Output_Printfc(bool bBold,COLORREF clrText,const char * pszString,...)
{
	va_list args;
	va_start(args,pszString);
	typedef void(*FnPtr)(PSMVERSION,bool,COLORREF,const char *,va_list);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Output_Printfc");
	(*pFunc)(MODIFIER_API_VERSION,bBold,clrText,pszString,args);
	va_end(args);	
}

inline const char * Psf_GetName(HPSF hFile)
{
	typedef const char *(*FnPtr)(PSMVERSION,HPSF);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Psf_GetName");
	return (*pFunc)(MODIFIER_API_VERSION,hFile);
}

inline bool Psf_Rename(HPSF hFile,const char * pszNewName)
{
	typedef bool(*FnPtr)(PSMVERSION,HPSF,const char *);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Psf_Rename");
	return (*pFunc)(MODIFIER_API_VERSION,hFile,pszNewName);
}

inline const char * Psf_GetExtension(HPSF hFile)
{
	typedef const char *(*FnPtr)(PSMVERSION,HPSF);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Psf_GetExtension");
	return (*pFunc)(MODIFIER_API_VERSION,hFile);
}

inline PSM_READONLY_MEMORY_BLOCK Psf_OpenOriginalFile(HPSF hFile)
{
	typedef PSM_READONLY_MEMORY_BLOCK(*FnPtr)(PSMVERSION,HPSF);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Psf_OpenOriginalFile");
	return (*pFunc)(MODIFIER_API_VERSION,hFile);
}

inline  void Psf_CloseOriginalFile(HPSF hFile,PSM_READONLY_MEMORY_BLOCK psmb)
{
	typedef void(*FnPtr)(PSMVERSION,HPSF,PSM_READONLY_MEMORY_BLOCK);
	FnPtr pFunc = (FnPtr)::GetProcAddress(GetModuleHandle(NULL),"Psf_CloseOriginalFile");
	(*pFunc)(MODIFIER_API_VERSION,hFile,psmb);
}

#endif

#endif  //_MODIFIERAPI_H_